#!/usr/bin/env python3
import sys
import syslog
import subprocess
import json
import re
import xml.etree.ElementTree as ET

DELIMITER = 100*'-'


def read_file_content_with_sudo(file_path):
  try:
    result = subprocess.run(['sudo', 'cat', file_path], capture_output=True, text=True, check=True)
    return result.stdout.strip()
  except subprocess.CalledProcessError as e:
    print(f"Error: {e}")
    return None


def converthtz(val):
    KHz = 1000
    MHz = 1000000
    GHz = 1000000000

    if val < KHz:
        return '{0}'.format(val,'Hz')
    elif KHz <= val < MHz:
        return '{0}KHz'.format(val/KHz)
    elif MHz <= val:
        return '{0}MHz'.format(val/MHz)


def convertbytes(B):
    KB = 1000
    MB = 1000000
    GB = 1000000000
    TB = 1000000000000

    if B < KB:
        return '{0} {1}'.format(B,'Bytes' if 0 == B > 1 else 'Byte')
    elif KB <= B < MB:
        return '{0}K'.format(B/KB)
    elif MB <= B < GB:
        return '{0}M'.format(B/MB)
    elif GB <= B < TB:
        return '{0}G'.format(B/GB)
    elif TB <= B:
        return '{0}T'.format(B/TB)


def convertfbytes(B):
  B = float(B)
  KB = float(1024)
  MB = float(KB ** 2) # 1,048,576
  GB = float(KB ** 3) # 1,073,741,824
  TB = float(KB ** 4) # 1,099,511,627,776

  if B < KB:
    return '{0} {1}'.format(B,'Bytes' if 0 == B > 1 else 'Byte')
  elif KB <= B < MB:
    return '{0:g}KiB'.format(B/KB)
  elif MB <= B < GB:
    return '{0:g}MiB'.format(B/MB)
  elif GB <= B < TB:
    return '{0:g}GiB'.format(B/GB)
  elif TB <= B:
    return '{0:g}TiB'.format(B/TB)


def show_hw_details():
  command = 'sudo /usr/bin/lshw -json -quiet'
  res = subprocess.check_output ( command , shell=True)
  result = res.decode('utf8')
  cfg = json.loads(result)
  print ("Hardware information")
  print (" class: " + cfg['id'])
  print (" description: " + cfg['description'])
  print (" product: " + cfg['product'])
  if 'vendor' in cfg:
      print (" vendor: " + cfg['vendor'])
  if 'version' in cfg:
    print (" version: " + cfg['version'])
  if 'serial' in cfg:
    print (" serial: " + cfg['serial'])
  print (" width: " + str(cfg['width']) + " bits")

  print (" capabilities: " + ", ".join(cfg['capabilities']))
  for childs in cfg['children']:
    print (" *-" + childs['id'])
    if 'description' in childs:
        print ("  description: " + childs['description'])
    if ('product' in childs):
      print ("  product: " + childs['product'])
    if ('vendor' in childs):
      print ("  vendor: " + childs['vendor'])
      print ("  physical id: " + childs['physid'])
    if ('logicalname' in childs):
      print ("  logical name: " + str(childs['logicalname']))
    if ('version' in childs):
      print ("  version: " + childs['version'])
    if 'serial' in childs:
      print ("  serial: " + childs['serial'])
    if ('size' in childs):
      print ("  size: " + convertbytes(childs['size']) + str(childs['units']))
    if ('slot' in childs):
      print ("  slot: " + childs['slot'])
      if ('capacity' in childs):
        if (childs['units'] == 'mWh'):
          print ("  capacity: " + str(childs['capacity']) + childs['units'])
        else:
          print ("  capacity: " + convertbytes(childs['capacity']) + childs['units'])
    if ('capabilities' in childs):
      print ("  capabilities: " + ', '.join(childs['capabilities']))

    config = "  configuration: "
    if ('configuration' in childs):
      if ('autonegotiation' in childs['configuration']):
        config = config + "autonegotiation=" + childs['configuration']['autonegotiation'] + " "
      if ('broadcast' in childs['configuration']):
        config = config + "broadcast=" + childs['configuration']['broadcast'] + " "
      if ('driver' in childs['configuration']):
        config = config + "driver=" + childs['configuration']['driver'] + " "
      if ('driverversion' in childs['configuration']):
        config = config + "driverversion=" + childs['configuration']['driverversion'] + " "
      if ('duplex' in childs['configuration']):
        config = config + "duplex=" + childs['configuration']['duplex'] + " "
      if ('link' in childs['configuration']):
        config = config + "link=" + childs['configuration']['link'] + " "
      if 'multicast' in childs['configuration']:
        config = config + "multicast=" + childs['configuration']['multicast'] + " "
      if ('port' in childs['configuration']):
        config = config + "port=" + childs['configuration']['port'] + " "
      if ('speed' in childs['configuration']):
        config = config + "speed=" + childs['configuration']['speed'] + " "
      print (config)

    if ('children' in childs):
      for child1 in childs['children']:
        print ("   *-" + child1['id'])
        print ("      class: " + child1['class'])
        if ('description' in child1):
          print ("      description: " + child1['description'])
        if ('product' in child1):
          print ("      product: " + child1['product'])
        if ('vendor' in child1):
          print ("      vendor: " + child1['vendor'])
        print ("      physical id: " + child1['physid'])
        if ('version' in child1):
          print ("      version: " + child1['version'])
          if ('date' in child1):
            print ("      date: " + child1['date'])
        if ('slot' in child1):
          print ("      slot: " + child1['slot'])
        if ('size' in child1):
          if (child1['id'] == "firmware" or child1['id'] == "memory" or 'cache' in child1['id']):
            print ("      size: " + convertfbytes(child1['size']))
          else:
            print ("      size: " + converthtz(child1['size']))
        if ('capacity' in child1):
          if (child1['id'] == "firmware" or 'cache' in child1['id']):
            print ("      capacity: " + convertbytes(child1['capacity']))
          else:
            print ("      capacity: " + converthtz(child1['capacity']))
        if ('width' in child1):
          print ("      width: " + str(child1['width']) + " bits")
        if ('clock' in child1):
          print ("      clock: " + converthtz(child1['clock']))
        if ('logicalname' in child1):
          print ("      logical name: " + child1['logicalname'])
        if ('capabilities' in child1):
          print  ("      capabilities: " + ', '.join(child1['capabilities']))

        config = "      configuration: "
        if ('configuration' in child1):
          if ('cores' in child1['configuration']):
            config = config + "cores=" +child1['configuration']['cores'] + " "
          if ('enabledcores' in child1['configuration']):
            config = config + "enabledcores=" +child1['configuration']['enabledcores'] + " "
          if ('threads' in child1['configuration']):
            config = config + "threads=" +child1['configuration']['threads'] + " "
          print (config)

        if ('children' in child1):
          for child2 in child1['children']:
            print ("       *-" + child2['id'])
            if ('description' in child2):
              print ("         description: " + child2['description'])
            if ('product' in child2):
              print ("         product: " + child2['product'])
            if ('vendor' in child2):
              print ("         vendor: " + child2['vendor'])
            print ("         physical id: " + child2['physid'])
            if ('serial' in child2):
              print ("         serial: " + child2['serial'])
            if ('slot' in child2):
              print ("         slot: " + child2['slot'])
            if ('size' in child2):
              print ("         size: " + convertfbytes(child2['size']))
            if ('capacity' in child2):
              print ("         capacity: " + convertbytes(child2['capacity']) + child2['units'])
            if ('width' in child2):
              print ("         width: " + str(child2['width']) + " bits")
            if ('clock' in child2):
              print ("         clock: " + converthtz(child2['clock']))
            if ('capabilities' in child2):
              print ("         capabilities: " + ', '.join(child2['capabilities']))

            config = "         configuration: "
            if ('configuration' in child2):
              if ('autonegotiation' in child2['configuration']):
                config = config + "autonegotiation=" + child2['configuration']['autonegotiation'] + " "
              if ('broadcast' in child2['configuration']):
                config = config + "broadcast=" + child2['configuration']['broadcast'] + " "
              if ('driver' in child2['configuration']):
                config = config + "driver=" + child2['configuration']['driver'] + " "
              if ('driverversion' in child2['configuration']):
                config = config + "driverversion=" + child2['configuration']['driverversion'] + " "
              if ('firmware' in child2['configuration']):
                config = config + "firmware=" + child2['configuration']['firmware'] + " "
              if ('latency' in child2['configuration']):
                config = config + "latency=" + child2['configuration']['latency'] + " "
              if ('link' in child2['configuration']):
                config = config + "link=" + child2['configuration']['link'] + " "
              if ('multicast' in child2['configuration']):
                config = config + "multicast=" + child2['configuration']['multicast'] + " "
              if ('port' in child2['configuration']):
                config = config + "port=" + child2['configuration']['port'] + " "
              if ('level' in child2['configuration']):
                config = config + "level=" + child2['configuration']['level'] + " "
              print (config)

            if ('children' in child2):
              for child3 in child2['children']:
                print ("        *-" + child3['id'])
                if ('description' in child3):
                  print ("          description: " + child3['description'])
                if ('product' in child3):
                  print ("          product: " + child3['product'])
                if ('vendor' in child3):
                  print ("          vendor: " + child3['vendor'])
                print ("          physical id: " + child3['physid'])
                if ('logicalname' in child3):
                  if isinstance(child3['logicalname'], list):
                    res = ""', '.join(child3['logicalname'])
                    print ("          logical name: " + res)
                  else:
                    print ("          logical name: " + str(child3['logicalname']))
                if ('version' in child3):
                  print ("          version: " + child3['version'])
                if ('serial' in child3):
                  print ("          serial: " + child3['serial'])
                if ('slot' in child3):
                  print ("          slot: " + child3['slot'])
                if ('size' in child3):
                  print ("          size: " + convertbytes(child3['size']))
                if ('capacity' in child3):
                  print ("          capacity: " + convertbytes(child3['capacity']))
                if ('width' in child3):
                  print ("          width: " + str(child3['width']) + " bits")
                if ('clock' in child3):
                  print ("          clock: " + converthtz(child3['clock']))
                if ('capabilities' in child3):
                  print  ("          capabilities: " + ', '.join(child3['capabilities']))

                config = "          configuration: "
                if ('configuration' in child3):
                  if ('autonegotiation' in child3['configuration']):
                    config = config + "autonegotiation=" + child3['configuration']['autonegotiation'] + " "
                  if ('broadcast' in child3['configuration']):
                    config = config + "broadcast=" + child3['configuration']['broadcast'] + " "
                  if ('driver' in child3['configuration']):
                    config = config + "driver=" + child3['configuration']['driver'] + " "
                  if ('slots' in child3['configuration']):
                    config = config + "slots=" + child3['configuration']['slots'] + " "
                  if ('speed' in child3['configuration']):
                    config = config + "speed=" + child3['configuration']['speed'] + " "
                  if ('driverversion' in child3['configuration']):
                    config = config + "driverversion=" + child3['configuration']['driverversion'] + " "
                  if ('firmware' in child3['configuration']):
                    config = config + "firmware=" + child3['configuration']['firmware'] + " "
                  if ('latency' in child3['configuration']):
                    config = config + "latency=" + child3['configuration']['latency'] + " "
                  if ('link' in child3['configuration']):
                    config = config + "link=" + child3['configuration']['link'] + " "
                  if ('multicast' in child3['configuration']):
                    config = config + "multicast=" + child3['configuration']['multicast'] + " "
                  if ('port' in child3['configuration']):
                    config = config + "port=" + child3['configuration']['port'] + " "
                  if ('created' in child3['configuration']):
                    config = config + "created=" + child3['configuration']['created'] + " "
                  if ('filesystem' in child3['configuration']):
                    config = config + "filesystem=" + child3['configuration']['filesystem'] + " "
                  if ('lastmountpoint' in child3['configuration']):
                    config = config + "lastmountpoint=" + child3['configuration']['lastmountpoint'] + " "
                  if ('modified' in child3['configuration']):
                    config = config + "modified=" + child3['configuration']['modified'] + " "
                  if ('mount.fstype' in child3['configuration']):
                    config = config + "mount.fstype=" + child3['configuration']['mount.fstype'] + " "
                  if ('mount.options' in child3['configuration']):
                    config = config + "mount.options=" + child3['configuration']['mount.options'] + " "
                  if ('mounted' in child3['configuration']):
                    config = config + "mounted=" + child3['configuration']['mounted'] + " "
                  if ('state' in child3['configuration']):
                    config = config + "state=" + child3['configuration']['state'] + " "
                  print (config)

                  if ('children' in child3):
                    for child4 in child3['children']:
                      print ("         *-" + child4['id'])
                      if ('description' in child4):
                        print ("           description: " + child4['description'])
                      if ('vendor' in child4):
                        print ("           vendor: " + child4['vendor'])
                      print ("           physical id: " + child4['physid'])
                      if ('version' in child4):
                        print ("           version: " + child4['version'])
                      if ('capabilities' in child4):
                        print ("           capabilities: " + ', '.join(child4['capabilities']))

                      config = "           configuration: "
                      if ('configuration' in child4):
                        if ('driver' in child4['configuration']):
                          config = config + "driver=" + child4['configuration']['driver'] + " "
                        if ('slots' in child4['configuration']):
                          config = config + "slots=" + child4['configuration']['slots'] + " "
                        if ('speed' in child4['configuration']):
                          config = config + "speed=" + child4['configuration']['speed'] + " "
                        print (config)


def get_cpu_details():
  try:
    result = subprocess.run(['lscpu'], stdout=subprocess.PIPE, text=True, check=True)
    cpu_info = {}
    for line in result.stdout.split('\n'):
      if ':' in line:
        key, value = line.split(':', 1)
        cpu_info[key.strip()] = value.strip()

    configuration = cpu_info.get('CPU(s)', 'Unknown')
    threads = cpu_info.get('Thread(s) per core', 'Unknown')

    if threads != 'Unknown':
      cpu_cores = f"{configuration}C / {int(configuration) * int(threads)}T"
    else:
      cpu_cores = f"{configuration}C"

    cpu_type = cpu_info.get('Model name', 'Unknown')
    return cpu_type, cpu_cores
  except subprocess.CalledProcessError as e:
    print("Error executing command:", e)
    return "Unknown", "Unknown"


def get_memory_info():
  try:
    output = subprocess.check_output(['sudo', 'dmidecode', '--type', '17']).decode('utf-8')
  except subprocess.CalledProcessError as e:
    print("Error executing dmidecode:", e)
    return None

  memory_info = []
  current_memory = {}

  for line in output.split('\n'):
    if line.startswith("Memory Device"):
      if current_memory:
        memory_info.append(current_memory)
      current_memory = {}
    elif line.strip() and ':' in line:
      key, value = line.split(":", 1)
      current_memory[key.strip()] = value.strip()

  if current_memory:
    memory_info.append(current_memory)
  return memory_info


def format_memory_info(memory_info):
  formatted_info = ""
  for mem_device in memory_info:
    if mem_device.get('Size') and mem_device.get('Type') and mem_device.get('Size') != 'No Module Installed':
      formatted_info += f" RAM Size\t\t: {mem_device['Size']}\n"
      formatted_info += f" RAM Type\t\t: {mem_device['Locator']} {mem_device['Type']} {mem_device.get('Type Detail', '')} {mem_device.get('Speed', '')}\n"
    elif mem_device.get('Size') == 'No Module Installed':
      formatted_info += f" RAM Type\t\t: {mem_device['Locator']} - Not installed\n"
  return formatted_info


def get_disk_details():
  try:
    result = subprocess.run(["sudo", "lshw", "-c", "disk", "-xml", "-notime", "-quiet"],
                            capture_output=True, check=True)
    xml_output = result.stdout.decode("utf-8")
    root = ET.fromstring(xml_output)
    disks = root.findall(".//node[@class='disk']")

    disk_details = []
    for i, disk in enumerate(disks, start=1):
      description_element = disk.find("./description")
      size_element = disk.find("./size")
      model_element = disk.find(".//product")  # Adjusted to search within the disk node
      if description_element is not None:
        description = description_element.text
      else:
        continue

      # Check if the disk has a size attribute
      if size_element is not None:
        size_bytes = int(size_element.text)
        size_gb = size_bytes / (1024 ** 3)
        size_gib = size_bytes / (1000 ** 3)
        size_info = f"Disk #{i} Size           : {size_gb:.9f}GB ({size_gib:.2f}GiB)\n"
      else:
        size_info = f"Disk #{i} Size           : Not Applicable\n"

      if model_element is not None:
        model = model_element.text
      else:
        model = "Unknown"

      disk_details.append(f" Disk #{i} Type           : {description}\n"
                          f" {size_info}"
                          f" Disk #{i} Model          : {model}\n"
                          f"{DELIMITER}")

    return disk_details

  except subprocess.CalledProcessError as e:
    print("Error executing command:", e)
    return []


def show_device():
  serial_number_path = '/config/.serial-number'
  hw_type_path = '/config/.hwtype'

  try:
    result = subprocess.run("sudo dmidecode | grep -A10 'System Information'",
                            shell=True,
                            check=True,
                            universal_newlines=True,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    product_name = re.search(r"Product Name: (.+)", result.stdout).group(1)
    hardware_type = read_file_content_with_sudo(hw_type_path)
    if not hardware_type:
      hardware_type = re.search(r"Manufacturer: (.+)", result.stdout).group(1)

    serial_number = read_file_content_with_sudo(serial_number_path)
    if not serial_number:
      serial_number = re.search(r"Serial Number: (.+)", result.stdout).group(1)

    sku_number = re.search(r"SKU Number: (.+)", result.stdout).group(1)
    output = DELIMITER + "\n"
    output += f" Device type            : {hardware_type.strip()} {product_name} ({sku_number})\n"
    output += f" Serial number          : {serial_number.strip()}\n"
    output += f" Hardware type          : {hardware_type.strip()}\n"
    output += DELIMITER
    return output
  except subprocess.CalledProcessError as e:
    return f"Error executing command: {e}"


def show_device_details():
  device = show_device()
  try:
    cpu_type, cpu_cores = get_cpu_details()

    memory_info = get_memory_info()
    ram_details = format_memory_info(memory_info) if memory_info else "RAM details could not be retrieved."
    disks_details = get_disk_details()

    print(device)
    output = f" CPU Type               : {cpu_type}\n"
    output += f" CPU Cores              : {cpu_cores}\n"
    output += DELIMITER + "\n"
    output += ram_details
    output += DELIMITER + "\n"
    output += "\n".join(disks_details)
    return output

  except subprocess.CalledProcessError as e:
    return f"Error executing command: {e}"


if __name__ == "__main__":
    argc = len(sys.argv)
    if 1 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!" )
        sys.exit(1)

    if sys.argv[1] == "hardware":
        show_hw_details()
        sys.exit(0)
    elif sys.argv[1] == "device":
        if argc == 3:
            device_details = show_device_details()
            print(device_details)
        else:
            device = show_device()
            print(device)
            sys.exit(0)
    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

sys.exit(0)
