#!/usr/bin/env python3

import sys
import syslog
import re
import os
import subprocess
import shlex
import signal
import xml.etree.ElementTree as ET
import vppapi_if.vppapi_if as vppif
from time import sleep


VPP_SH_IFACE_COMMAND = 'sudo /usr/bin/vppctl show interface '
CLASS_NET_COMMAND = 'sudo cat /sys/class/net/'
vpp_if = vppif.VppApiInterface('show-commands')


PACKET_LABELS_STATISTICS = {
    "packets": "Packets",
    "bytes": "Bytes",
    "error": "Errors",
    "drops": "Drops",
    "frame drops": "Frame Drops",
    "oversize": "Oversize",
    "crs errors": "CRC errors",
    "collisions": "Collisions",
}


def signal_handler(_, __):
    print('\nTerminated by the user!')
    sys.exit(1)  # Exit the program with a status code of 1


def get_all_intf_names():
    try:
        output = subprocess.check_output(['sysrepocfg', '--datastore=running', '--format=xml',
                                          "--xpath=/ucpe:config/interfaces:interface/name",
                                          '--export']).decode().strip()
        root = ET.fromstring(output)
        namespaces = {'ucpe': 'urn:ucpe', 'interfaces': 'urn:interfaces'}
        intf_names = [elem.text.strip() for elem in root.findall(".//{urn:interfaces}name", namespaces)]
        return intf_names

    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f"Failed to obtain interface names. Error: {e}.")
        return []


def get_intf_state(output, intf):
    if intf.startswith('tap'):
        state_matches = re.findall(r'\b(up|down)\b', output)
        if state_matches:
            state = "up" if "up" in state_matches else "down"
        else:
            state = "down"
    else:

        ifconfig_process = subprocess.run(['/sbin/ifconfig', intf], capture_output=True, text=True)
        if ifconfig_process.returncode == 0:
            if "UP" in ifconfig_process.stdout:
                state = "up"
            else:
                state = "down"
        else:
            state = "unknown"

    return state


def get_intf_link_state(intf):
    # Check if carrier file exists
    carrier_path = f"/sys/class/net/{intf}/carrier"
    if os.path.exists(carrier_path):
        carrier_output = subprocess.run(['cat', carrier_path], capture_output=True, text=True)
        if carrier_output.returncode == 0:
            carrier_state = carrier_output.stdout.strip()
            if carrier_state == "1":
                state = 'up'
            else:
                state = 'down'
        else:
            state = 'unknown'
    else:
        state = 'unknown'

    return state


def get_mac_address(intf):
    if intf.startswith('tap') or intf.startswith('VirtualEthernet'):
        mac_command = (f'sudo /usr/bin/vppctl show hardware-interface {intf} '
                       f'| grep "Ethernet addr" |  tr -s " " | cut -f4 -d " " | cut -c 1-17')
        try:
            output = subprocess.check_output(mac_command, shell=True, stderr=subprocess.PIPE)
            mac_address = output.decode('utf-8').strip()
        except subprocess.CalledProcessError as e:
            if "No such file or directory" in e.stderr.decode('utf-8'):
                syslog.syslog(syslog.LOG_ERR, f'File not found for MAC address of {intf}')
                mac_address = 'unknown'
            else:
                syslog.syslog(syslog.LOG_ERR, f'Can not obtain the MAC address for {intf} due to {e}')
                mac_address = 'unknown'
    else:
        try:
            mac_output = subprocess.check_output(shlex.split(CLASS_NET_COMMAND + intf + '/address'),
                                                 stderr=subprocess.PIPE).decode('utf-8')
            mac_address = mac_output.strip()
        except subprocess.CalledProcessError as e:
            if "No such file or directory" in e.stderr.decode('utf-8'):
                syslog.syslog(syslog.LOG_ERR, f'File not found for MAC address of {intf}')
                mac_address = 'unknown'
            else:
                syslog.syslog(syslog.LOG_ERR, f'Can not obtain the MAC address for {intf} due to {e}')
                mac_address = 'unknown'
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, f'Error occurred while getting MAC address for {intf}: {e}')
            mac_address = 'unknown'
    return mac_address


def get_intf_mode(intf):
    try:
        output = subprocess.check_output(['sysrepocfg', '--datastore=running', '--format=xml',
                                          f"--xpath=/ucpe:config/interfaces:interface[name='{intf}']/mode",
                                          '--export']).decode().strip()
        root = ET.fromstring(output)
        namespaces = {'ucpe': 'urn:ucpe', 'interfaces': 'urn:interfaces'}
        intf_mode = root.find(".//{urn:interfaces}mode", namespaces).text.strip()
        return intf_mode

    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f"The interface mode can not be obtained. Error: {e}.")
        return None


def get_statistics_output(intf):
    if intf.startswith('data'):
        intf_mode = get_intf_mode(intf)
        if intf_mode == 'virtio':
            stat_interface = f'host-{intf}'
        else:
            stat_interface = intf
    elif any(intf.startswith(prefix) for prefix in ['tun', 'wifi', 'lte']):
        stat_interface = intf
    elif intf.startswith('tap') or intf.startswith('VirtualEthernet'):
        stat_interface = intf
    else:
        stat_interface = vpp_if.get_tap_interface_by_host_ifname(intf).get('ifname')

    stat_output = subprocess.check_output(shlex.split(VPP_SH_IFACE_COMMAND + stat_interface)).decode('utf-8')
    return stat_output


def extract_statistics_per_statistics_command(output):
    regex_patterns = {
        'rx_packets': r"rx packets\s+(\d+)",
        'rx_bytes': r"rx bytes\s+(\d+)",
        'tx_packets': r"tx packets\s+(\d+)",
        'tx_bytes': r"tx bytes\s+(\d+)",
        'rx_errors': r"rx-error\s+(\d+)",
        'tx_errors': r"tx-error\s+(\d+)",
        'drops': r"drops\s+(\d+)",
        'frame_drops': r"frame drops\s+(\d+)",
        'oversize': r"oversize\s+(\d+)",
        'crs_errors': r"crs errors\s+(\d+)",
        'collisions': r"collisions\s+(\d+)"
    }

    matches = {key: re.search(pattern, output) for key, pattern in regex_patterns.items()}
    extracted_values = {key: match.group(1) if match else 0 for key, match in matches.items()}

    return tuple(extracted_values.values())


def collect_interface_data_for_statistics(intf):
    intf_statistics = {}
    name = intf
    statistics_output = get_statistics_output(intf)

    syslog.syslog(syslog.LOG_ERR, f'The statistic output is {statistics_output}')

    mac = get_mac_address(intf)

    state = get_intf_state(statistics_output, intf)
    link_state = get_intf_link_state(intf)

    stats = {}
    statistics = extract_statistics_per_statistics_command(statistics_output)
    statistics_names = [
        'rx_packets', 'rx_bytes', 'tx_packets', 'tx_bytes',
        'rx_errors', 'tx_errors', 'drops', 'frame_drops',
        'oversize', 'crs_errors', 'collision'
    ]

    for index, value in enumerate(statistics):
        stats[statistics_names[index]] = int(value) if value is not None else 0

    intf_statistics[intf] = {
        'name': name,
        'mac': mac,
        'state': state,
        'link state': link_state,
        'stats': stats,
    }

    return intf_statistics


def print_statistics_per_interface_normal(stats, intf):
    if intf not in stats:
        print(f"No statistics found for interface {intf}")
        return

    interface_stats = stats[intf]
    print('-' * 80)
    print(f"Interface: {interface_stats.get('name', '-')}, MAC: {interface_stats.get('mac', '-')}")
    print(f"admin state: {interface_stats.get('state', '-')}, link state: {interface_stats.get('link state', '-')}")
    print('-' * 80)
    print('{:>52} {:>19}'.format('rx', 'tx'))
    print('-' * 80)

    for label, print_label in PACKET_LABELS_STATISTICS.items():
        value = interface_stats['stats'].get(label)  # Get the value or None if not found
        if value is None:  # If value is None, assign an empty string
            value = 0

        if label in ['packets', 'bytes', 'error']:

            rx_stat_value = interface_stats['stats'].get(f"rx_{label}", '-')
            tx_stat_value = interface_stats['stats'].get(f"tx_{label}", '-')
            print(f"{print_label:<50}{rx_stat_value:<20}{tx_stat_value}")
        else:
            print(f"{print_label:<50}{value:<20}{'-'}")


def print_statistics_per_interface_csv(stats, intf):
    data = [
        stats[intf]['name'], stats[intf]['mac'], stats[intf]['state'], stats[intf]['link state'],
        *[stats[intf]['stats'][key] for key in ['rx_packets', 'rx_bytes', 'tx_packets', 'tx_bytes',
                                          'rx_errors', 'tx_errors', 'drops', 'frame_drops', 'oversize',
                                                'crs_errors', 'collision']]
    ]

    print(','.join(map(str, [d for d in data if d != ''])))


def display_data_for_statistics(intf, formatting='normal'):
    statistics = collect_interface_data_for_statistics(intf)
    if formatting == 'normal':
        print_statistics_per_interface_normal(statistics, intf)
    elif formatting == 'csv':
        print_statistics_per_interface_csv(statistics, intf)


if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal_handler)
    argc = len(sys.argv)
    all_interfaces = get_all_intf_names()

    command = sys.argv[1]
    if command == 'statistics':
        interface_name = sys.argv[2]
        output_format = "csv" if argc > 3 and sys.argv[3] == "csv" else "normal"
        display_data_for_statistics(interface_name, output_format)

