#!/usr/bin/env bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
source /scripts/basic_vars
set -o nounset
set -o pipefail
set -o errexit
#
appname=""
#
function onboard() {
  if [ -z "${is_native_applications_enabled_in_config}" ]; then
    echo "Native applications are disabled in configuration!"
    exit 3
  fi
  if [ ! -f ${VERIFIED_LICENSE_FILE} ]; then
    echo "No valid license !"
    exit 4
  fi
  if [ -z "${is_native_app_enabled_in_license}" ]; then
    echo "Native applications not allowed by a valid license !";
    exit 5
  fi
  if [ ! -f /home/admin/user_files/${appname}.tar.gz ]; then
    echo "Missing ${appname}.tar.gz ! Upload it first!";
    exit 0;
  fi
  /scripts/application_onboard.sh ${appname}
} # function onboard
#
function remove() {
  shopt -s nocasematch
  if [ -n "$(echo ${appname} | grep --line-regexp clavister)" ]; then
    echo "Removing Clavister will cause system reboot !"
    echo -n "The command requires system reboot, proceed(yes/no)? "
    read ANS
    if [ "$ANS" != "YES" -a "$ANS" != "yes" ]; then
      echo "Aborted."
      exit 0
    fi
    touch /run/.reboot_in_progress
  fi
  if [ -n "$(echo ${appname} | grep --line-regexp clavister)" ]; then
    systemctl stop sshd 1>/dev/null
    systemctl stop ssh-root 1>/dev/null
  fi
  shopt -u nocasematch
  if test -f "/lib/systemd/system/$(echo -n "${appname}" | tr '[:upper:]' '[:lower:]').service"; then
    systemctl disable $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]').service 2>/dev/null
    systemctl --no-block stop $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]').service
  fi
  rm -rf /root/apps/$(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')
  rm -f /lib/systemd/system/$(echo -n "${appname}" | tr '[:upper:]' '[:lower:]').service
  systemctl daemon-reload
  shopt -s nocasematch
  if [ -n "$(echo ${appname} | grep --line-regexp clavister)" ]; then
    rm -rf /config/_reboot/tve/now
    mkdir -p /config/_reboot/tve/now
  fi
  shopt -u nocasematch
} # function remove
#
function start() {
  if [ -z "${is_native_applications_enabled_in_config}" ]; then
    echo "Native applications are disabled in configuration!"
    exit 3
  fi
  if [ ! -f ${VERIFIED_LICENSE_FILE} ]; then
    echo "No valid license !"
    exit 4
  fi
  if [ -z "${is_native_app_enabled_in_license}" ]; then
    echo "Native applications not allowed by a valid license !";
    exit 5
  fi
  if [ -n "$(echo ${appname} | grep --ignore-case clavister)" ] \
     && [ -z "${is_native_app_clavister_enabled_in_license}" ]; then
    echo "Native application Clavister not allowed by a valid license !"
    exit 5
  elif [ -n "$(echo ${appname} | grep --ignore-case mikrotik)" ] \
       && [ -z "${is_native_app_mikrotik_enabled_in_license}" ]; then
    echo "Native application MikroTik not allowed by a valid license !"
    exit 5
  fi

  /scripts/application_start.sh $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')
} # function start
#
function stop() {
  shopt -s nocasematch
  if [ -n "$(echo ${appname} | grep --line-regexp clavister)" ]; then
    systemctl stop sshd 1>/dev/null
    systemctl stop ssh-root 1>/dev/null
  fi
  shopt -u nocasematch
  /scripts/application_stop.sh $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')
  if [ -n "$(echo ${appname} | grep --line-regexp clavister)" ]; then
    rm -rf /config/_reboot/tve/now
    mkdir -p /config/_reboot/tve/now
  fi
} # function stop
#
function list() {
  AUUID="$(cat /proc/sys/kernel/random/uuid)"
  /scripts/application_list.sh all $AUUID || true
  if [ -f /tmp/application_list_$AUUID.json ]; then
    echo -e "application\tis-active\tversion" | column -t
    echo "---------------------------------------"
    /usr/bin/jq -r '.list[] | [.app,."is-active",.version|@json] | @tsv' /tmp/application_list_$AUUID.json | column -t
    rm -f /tmp/application_list_$AUUID.json || true
  fi
} # function list
#
function console() {
  if [ -z "${is_native_applications_enabled_in_config}" ]; then
    echo "Native applications are disabled in configuration!"
    exit 3
  fi
  if [ ! -f ${VERIFIED_LICENSE_FILE} ]; then
    echo "No valid license !"
    exit 4
  fi
  if [ -z "${is_native_app_enabled_in_license}" ]; then
    echo "Native applications not allowed by a valid license !";
    exit 5
  fi
  if test -d "/root/apps/$(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')"; then
    shopt -s nocasematch
    if [[ "${appname}" == "slmprobe" ]]; then
      echo "SLMProbe console access is not available"
    else
      /root/apps/$(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')/console.sh
    fi
  else
    echo "Missing $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]') !";exit 0;
  fi
} # function console
#
function retrieve() {
  if [ -z "${is_native_applications_enabled_in_config}" ]; then
    echo "Native applications are disabled in configuration!"
    exit 3
  fi
  if [ ! -f ${VERIFIED_LICENSE_FILE} ]; then
    echo "No valid license !"
    exit 4
  fi
  if [ -z "${is_native_app_enabled_in_license}" ]; then
    echo "Native applications not allowed by a valid license !";
    exit 5
  fi
  if test -d "/root/apps/$(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')"; then
    /scripts/application_retrieve.sh $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')
  else
    echo "Missing $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]') !";exit 0;
  fi
} # function retrieve
#
function reset() {
  if [ -z "${is_native_applications_enabled_in_config}" ]; then
    echo "Native applications are disabled in configuration!"
    exit 3
  fi
  if [ ! -f ${VERIFIED_LICENSE_FILE} ]; then
    echo "No valid license !"
    exit 4
  fi
  if [ -z "${is_native_app_enabled_in_license}" ]; then
    echo "Native applications not allowed by a valid license !";
    exit 5
  fi
  if [ -n "$(echo ${appname} | grep --ignore-case clavister)" ] \
     && [ -z "${is_native_app_clavister_enabled_in_license}" ]; then
    echo "Native application Clavister not allowed by a valid license !"
    exit 5
  elif [ -n "$(echo ${appname} | grep --ignore-case mikrotik)" ] \
       && [ -z "${is_native_app_mikrotik_enabled_in_license}" ]; then
    echo "Native application MikroTik not allowed by a valid license !"
    exit 5
  fi

  /scripts/application_reset.sh $(echo -n "${appname}" | tr '[:upper:]' '[:lower:]')
} # function reset
#
operation=""
while [[ "$#" -gt 0 ]]; do
  case $1 in
    --app) appname="$2"; shift;;
    --onboard) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --remove) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --start) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --stop) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --retrieve) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --console) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --list) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    --reset) if [ -n "${2}" ]; then operation="$2"; fi; shift;;
    *) echo "Unknown parameter passed: $1"; exit 1;;
  esac
  shift
done
eval ${operation}
exit 0
